@@ -25,9 +25,10 @@ produce an MkDocs-compatible folder structure. """
 from nr.databind.core import Field, Struct, ProxyType
 from nr.interface import implements, override
 from pydoc_markdown.contrib.renderers.markdown import MarkdownRenderer
-from pydoc_markdown.interfaces import Renderer
+from pydoc_markdown.interfaces import Renderer, Resolver, Server
 from pydoc_markdown.reflection import Object, ModuleGraph
-from typing import Dict, Iterable, List, Tuple
+from pydoc_markdown.util.pages import Page, Pages
+from typing import Dict, Iterable, List, Optional, Tuple
 import copy
 import fnmatch
 import logging
@@ -38,7 +39,6 @@ import subprocess
 import yaml
 
 logger = logging.getLogger(__name__)
-Page = ProxyType()
 
 
 class CustomizedMarkdownRenderer(MarkdownRenderer):
@@ -47,24 +47,7 @@ class CustomizedMarkdownRenderer(MarkdownRenderer):
   render_toc = Field(bool, default=False)
 
 
-@Page.implementation  # pylint: disable=function-redefined
-class Page(Struct):
-  """ Desribes a page that is rendered by the #MkdocsRenderer. """
-
-  name = Field(str, default=None)
-  title = Field(str)
-  href = Field(str, default=None)
-  source = Field(str, default=None)
-  contents = Field([str], default=None)
-  children = Field([Page], default=list)
-
-  def get_name(self):
-    if self.name:
-      return self.name
-    return re.sub(r'\s+', '-', self.title.lower())
-
-
-@implements(Renderer)
+@implements(Renderer, Server)
 class MkdocsRenderer(Struct):
   #: The output directory for the generated Markdown files. Defaults to
   #: `build/docs`.
@@ -74,7 +57,7 @@ class MkdocsRenderer(Struct):
   clean_docs_directory_on_render = Field(bool, default=True)
 
   #: The pages to render into the output directory.
-  pages = Field([Page])
+  pages = Field(Pages)
 
   #: Markdown renderer settings.
   markdown = Field(CustomizedMarkdownRenderer, default=CustomizedMarkdownRenderer)
@@ -91,37 +74,6 @@ class MkdocsRenderer(Struct):
   def docs_dir(self) -> str:
     return os.path.join(self.output_directory, 'docs')
 
-  def organize_modules(self, graph) -> Iterable[Tuple[List[str], Page, List[Object]]]:
-    """ Organizes the objects in the *graph* in pairs with the pages that
-    are supposed to contain them (per the "contents" filter). """
-
-    def _match(page, node):
-      if not page.contents:
-        return False
-      path = node.path()
-      return any(fnmatch.fnmatch(path, x) for x in page.contents or ())
-
-    def _visit(page, path):
-      def _update_nodes(x):
-        x.visible = x.visible and _match(page, x)
-        # Make sure all parents are visible as well.
-        while x and x.visible:
-          x = x.parent
-          if x:
-            x.visible = True
-      clone = copy.deepcopy(graph)
-      clone.visit(_update_nodes)
-      yield path, page, clone.modules
-      path = path + [page.get_name()]
-      for sub_page in page.children:
-        yield from _visit(sub_page, path)
-
-    for page in self.pages:
-      yield from _visit(page, [])
-
-  def mkdocs_serve(self):
-    return subprocess.Popen(['mkdocs', 'serve'], cwd=self.output_directory)
-
   def generate_mkdocs_nav(self, page_to_filename: Dict[Page, str]) -> Dict:
     def _generate(pages):
       result = []
@@ -139,32 +91,21 @@ class MkdocsRenderer(Struct):
   # Renderer
 
   @override
-  def render(self, graph):
+  def render(self, graph: ModuleGraph) -> None:
     if self.clean_docs_directory_on_render and os.path.isdir(self.docs_dir):
       logger.info('Cleaning directory "%s"', self.docs_dir)
       shutil.rmtree(self.docs_dir)
 
     page_to_filename = {}
 
-    for path, page, items in self.organize_modules(graph):
-      # Construct the filename for the generated Markdown page.
-      path = path + [page.get_name()]
-      if page.children:
-        if not page.contents and not page.source:
-          continue
-        path.append('index')
-      filename = os.path.join(self.docs_dir, *path) + '.md'
-      page_to_filename[id(page)] = filename
+    for item in self.pages.iter_hierarchy():
+      filename = item.filename(self.docs_dir, '.md')
+      if not filename:
+        continue
 
-      # Render the page or copy from the specified source file.
-      os.makedirs(os.path.dirname(filename), exist_ok=True)
-      if page.source:
-        logger.info('Writing "%s" (source: "%s")', filename, page.source)
-        shutil.copyfile(page.source, filename)
-      else:
-        logger.info('Rendering "%s"', filename)
-        self.markdown.filename = filename
-        self.markdown.render(ModuleGraph(items))
+      page_to_filename[id(page)] = filename
+      self.markdown.filename = filename
+      item.page.render(filename, graph, self.markdown)
 
     config = copy.deepcopy(self.mkdocs_config)
     if self.site_name:
@@ -180,5 +121,23 @@ class MkdocsRenderer(Struct):
       yaml.dump(config, fp)
 
   @override
-  def get_resolver(self, graph):
+  def get_resolver(self, graph: ModuleGraph) -> Optional[Resolver]:
+    # TODO (@NiklasRosenstein): The resolver returned by the Markdown
+    #   renderer does not implement linking across multiple pages.
     return self.markdown.get_resolver(graph)
+
+  # Server
+
+  @override
+  def get_server_url(self) -> str:
+    return 'http://localhost:8000'
+
+  @override
+  def start_server(self) -> subprocess.Popen:
+    return subprocess.Popen(['mkdocs', 'serve'], cwd=self.output_directory)
+
+  @override
+  def reload_server(self, process: subprocess.Popen):
+    # While MkDocs does support file watching and automatic reloading,
+    # it appears to be a bit quirky. Let's just restart the whole process.
+    process.terminate()
